@@ -23,6 +23,7 @@ from typing import (
 from . import metrics
 from . import parser
 from . import pings
+from . import tags
 from . import util
 
 
@@ -234,6 +235,16 @@ def check_misspelled_pings(
                 yield f"Ping '{ping}' seems misspelled. Did you mean '{builtin}'?"
 
 
+def check_tags_required(
+    metric_or_ping: Union[metrics.Metric, pings.Ping], parser_config: Dict[str, Any]
+) -> LintGenerator:
+
+    if parser_config.get("require_tags", False) and not len(
+        metric_or_ping.metadata.get("tags", [])
+    ):
+        yield "Tags are required but no tags specified"
+
+
 def check_user_lifetime_expiration(
     metric: metrics.Metric, parser_config: Dict[str, Any]
 ) -> LintGenerator:
@@ -303,6 +314,7 @@ METRIC_CHECKS: Dict[
     "BUG_NUMBER": (check_bug_number, CheckType.error),
     "BASELINE_PING": (check_valid_in_baseline, CheckType.error),
     "MISSPELLED_PING": (check_misspelled_pings, CheckType.error),
+    "TAGS_REQUIRED": (check_tags_required, CheckType.error),
     "EXPIRATION_DATE_TOO_FAR": (check_expired_date, CheckType.warning),
     "USER_LIFETIME_EXPIRATION": (check_user_lifetime_expiration, CheckType.warning),
     "EXPIRED": (check_expired_metric, CheckType.warning),
@@ -315,6 +327,7 @@ PING_CHECKS: Dict[
     str, Tuple[Callable[[pings.Ping, dict], LintGenerator], CheckType]
 ] = {
     "BUG_NUMBER": (check_bug_number, CheckType.error),
+    "TAGS_REQUIRED": (check_tags_required, CheckType.error),
     "REDUNDANT_PING": (check_redundant_ping, CheckType.error),
 }
 
@@ -333,10 +346,32 @@ class GlinterNit:
         )
 
 
+def _lint_item_tags(
+    item_name: str,
+    item_type: str,
+    item_tag_names: List[str],
+    valid_tag_names: List[str],
+) -> List[GlinterNit]:
+    invalid_tags = [tag for tag in item_tag_names if tag not in valid_tag_names]
+    return (
+        [
+            GlinterNit(
+                "INVALID_TAGS",
+                item_name,
+                f"Invalid tags specified in {item_type}: {', '.join(invalid_tags)}",
+                CheckType.error,
+            )
+        ]
+        if len(invalid_tags)
+        else []
+    )
+
+
 def _lint_pings(
-    category: Dict[str, Union[metrics.Metric, pings.Ping]],
+    category: Dict[str, Union[metrics.Metric, pings.Ping, tags.Tag]],
     parser_config: Dict[str, Any],
-):
+    valid_tag_names: List[str],
+) -> List[GlinterNit]:
     nits: List[GlinterNit] = []
 
     for (ping_name, ping) in sorted(list(category.items())):
@@ -354,7 +389,14 @@ def _lint_pings(
                         )
                         for msg in new_nits
                     )
-
+        nits.extend(
+            _lint_item_tags(
+                ping_name,
+                "ping",
+                ping.metadata.get("tags", []),
+                valid_tag_names,
+            )
+        )
     return nits
 
 
@@ -374,12 +416,17 @@ def lint_metrics(
         parser_config = {}
 
     nits: List[GlinterNit] = []
+    valid_tag_names = [tag for tag in objs.get("tags", [])]
     for (category_name, category) in sorted(list(objs.items())):
         if category_name == "pings":
-            nits.extend(_lint_pings(category, parser_config))
+            nits.extend(_lint_pings(category, parser_config, valid_tag_names))
             continue
 
-        # Make sure the category has only Metrics, not Pings
+        if category_name == "tags":
+            # currently we have no linting for tags
+            continue
+
+        # Make sure the category has only Metrics, not Pings or Tags
         category_metrics = dict(
             (name, metric)
             for (name, metric) in category.items()
@@ -411,6 +458,16 @@ def lint_metrics(
                             for msg in new_nits
                         )
 
+            # also check that tags for metric are valid
+            nits.extend(
+                _lint_item_tags(
+                    ".".join([metric.category, metric.name]),
+                    "metric",
+                    metric.metadata.get("tags", []),
+                    valid_tag_names,
+                )
+            )
+
     if len(nits):
         print("Sorry, Glean found some glinter nits:", file=file)
         for nit in nits:
@@ -431,7 +488,7 @@ def lint_metrics(
 def lint_yaml_files(
     input_filepaths: Iterable[Path],
     file=sys.stderr,
-    parser_config: Dict[str, Any] = None,
+    parser_config: Optional[Dict[str, Any]] = None,
 ) -> List:
     """
     Performs glinter YAML lint on a set of files.
@@ -486,17 +543,21 @@ def glinter(
     if parser_config is None:
         parser_config = {}
 
-    if lint_yaml_files(input_filepaths, file=file, parser_config=parser_config):
-        return 1
+    errors = 0
+
+    nits = lint_yaml_files(input_filepaths, file=file, parser_config=parser_config)
+    errors += len(nits)
 
     objs = parser.parse_objects(input_filepaths, parser_config)
-
-    if util.report_validation_errors(objs):
-        return 1
+    errors += util.report_validation_errors(objs)
 
     nits = lint_metrics(objs.value, parser_config=parser_config, file=file)
-    if any(nit.check_type == CheckType.error for nit in nits):
-        return 1
-    if len(nits) == 0:
+    errors += len([nit for nit in nits if nit.check_type == CheckType.error])
+
+    if errors == 0:
         print("✨ Your metrics are Glean! ✨", file=file)
-    return 0
+        return 0
+
+    print(f"❌ Found {errors} errors.")
+
+    return 1
