@@ -13,6 +13,7 @@ from typing import Any, Dict, List, Optional, Type, Union  # noqa
 
 
 from . import pings
+from . import tags
 from . import util
 
 
@@ -47,6 +48,7 @@ class Metric:
         description: str,
         notification_emails: List[str],
         expires: Any,
+        metadata: Optional[Dict] = None,
         data_reviews: Optional[List[str]] = None,
         version: int = 0,
         disabled: bool = False,
@@ -58,7 +60,7 @@ class Metric:
         data_sensitivity: Optional[List[str]] = None,
         defined_in: Optional[Dict] = None,
         telemetry_mirror: Optional[str] = None,
-        _config: Dict[str, Any] = None,
+        _config: Optional[Dict[str, Any]] = None,
         _validated: bool = False,
     ):
         # Avoid cyclical import
@@ -71,6 +73,9 @@ class Metric:
         self.description = description
         self.notification_emails = notification_emails
         self.expires = expires
+        if metadata is None:
+            metadata = {}
+        self.metadata = metadata
         if data_reviews is None:
             data_reviews = []
         self.data_reviews = data_reviews
@@ -193,10 +198,16 @@ class Metric:
         return self.disabled or self.is_expired()
 
     def is_expired(self) -> bool:
-        return self._config.get("custom_is_expired", util.is_expired)(self.expires)
+        def default_handler(expires) -> bool:
+            return util.is_expired(expires, self._config.get("expire_by_version"))
+
+        return self._config.get("custom_is_expired", default_handler)(self.expires)
 
     def validate_expires(self):
-        return self._config.get("custom_validate_expires", util.validate_expires)(
+        def default_handler(expires):
+            return util.validate_expires(expires, self._config.get("expire_by_version"))
+
+        return self._config.get("custom_validate_expires", default_handler)(
             self.expires
         )
 
@@ -353,6 +364,30 @@ class Jwe(Metric):
         )
 
 
+class CowString(str):
+    """
+    Wrapper class for strings that should be represented
+    as a `Cow<'static, str>` in Rust,
+    or `String` in other target languages.
+
+    This wraps `str`, so unless `CowString` is specifically
+    handled it acts (and serializes)
+    as a string.
+    """
+
+    def __init__(self, val: str):
+        self.inner: str = val
+
+    def __eq__(self, other):
+        return self.inner == other.inner
+
+    def __hash__(self):
+        return self.inner.__hash__()
+
+    def __lt__(self, other):
+        return self.inner.__lt__(other.inner)
+
+
 class Labeled(Metric):
     labeled = True
 
@@ -360,7 +395,7 @@ class Labeled(Metric):
         labels = kwargs.pop("labels", None)
         if labels is not None:
             self.ordered_labels = labels
-            self.labels = set(labels)
+            self.labels = set([CowString(label) for label in labels])
         else:
             self.ordered_labels = None
             self.labels = None
@@ -396,4 +431,14 @@ class Rate(Metric):
         super().__init__(*args, **kwargs)
 
 
-ObjectTree = Dict[str, Dict[str, Union[Metric, pings.Ping]]]
+class Denominator(Counter):
+    typename = "denominator"
+    # A denominator is a counter with an additional list of numerators.
+    numerators: List[Rate] = []
+
+
+class Text(Metric):
+    typename = "text"
+
+
+ObjectTree = Dict[str, Dict[str, Union[Metric, pings.Ping, tags.Tag]]]
